package Trap::SAMFS;
#<copyright>
# ----------------------------------------------------------
# Sun Proprietary/Confidential Code
# Copyright 2001, Sun Microsystems, Inc. All rights reserved.
# ----------------------------------------------------------
#</copyright>
#  $Name:  $ 
#  $Id: SAMFS.pm,v 1.8 2004/06/03 01:32:18 bkeslin Exp $

use strict qw(vars);
use State;
use PDM::Parse;
use Report;
use Policies;
use Trap;


sub REGISTER {
   return ['.1.3.6.1.4.1.42.2.28.21.1'];
}

use vars qw(@SAM_SEV  %SAM_MAP);

@SAM_SEV = ('Unusable','Alert','Critical','Error','Warning', 'Notice', 'Information');

%SAM_MAP = ('2.1.1' => 'trap_type',
            '2.1.2' => 'severity',
            '2.1.5' => 'systemID',
            '2.1.3' => 'msg',
            '2.1.6' => 'date',
            '2.2.9' => 'details',
           );


sub RUN {
 Debug->print3("->>>Running SAMFS Trap Agent");
  my($trap_agent, $lines, $errors) = @_;

  my $renv   = System->get_renv();
  my $Config = PDM::ConfigFile->read();
  my(%DEV, $x);
  my $policies = Policies->new("samfs_policies");

  my $devs = $Config->devices();
  foreach my $d (@$devs) {
     $DEV{$d->{ipno}} = $d;
  }

  for ($x = 0; $x <= $#$lines; $x++ ) {
     my $line = Trap->last_snmp($lines->[$x], 3);
     my $sev  = $line->{'2.1.2'};
     my $topic= $line->{'2.1.1'};
     my $id   = $line->{'2.1.5'};
     my $desc = $line->{'2.1.3'};
     my $ip   = $line->{IP};
     my $dev_name=$DEV{$ip}{name};
     my $key  = $DEV{$ip}{key};
     my ($i, $data);
     for ($i=1; $i <= 7; $i++) {
       if ( $line->{"9.1.$i"}) {
         $data .= $line->{"9.1.$i"} . ",";
       }
     }

     my $loglines = ["$line->{DATE} $line->{TIME} $line->{DEVICE} $ip Sev:$sev Type:$topic SystemID:$id $desc: $data"];

     my $y = 0;
     my $key = "samfs:$key:$topic";
     my $key0= "samfs:$key";
     $policies->run($errors, 
                    $loglines, \$y, 
                    $topic,
                    $dev_name, 
                    $key, 
                    $ip,
                    $key0, 
                    1);
  }
}



#############################################
# $lines is an array of hashes is a hash :
#
#  {   '.iso.org.dod.internet.snmpV2.snmpModules.snmpMIB.snmpMIBObjects.snmpTrap.snmpTrapOID.0' => 'OID: enterprises.42.2.28.21.1.2.2.9.0.1'
#      'DATE' => '2003-11-22'
#      'DEVICE' => 'ns-east-107.East.Sun.COM'
#      'IP' => '10.8.11.107'
#      'TIME' => '10:32:03'
#      'enterprises.42.2.28.21.1.2.1.1' => 'TapeAlert'
#      'enterprises.42.2.28.21.1.2.1.2' => 4
#      'enterprises.42.2.28.21.1.2.1.3' => 'The tape drive is having problems reading data. No data has been.lost, but there has been a reduction in the performance of the.tape.'
#      'enterprises.42.2.28.21.1.2.1.5' => 'ns-east-107'
#      'enterprises.42.2.28.21.1.2.1.6' => 'Thu Nov 20 12:37:54 2003.'
#      'enterprises.42.2.28.21.1.2.2.9.1.1' => 'SSC2 1'
#      'enterprises.42.2.28.21.1.2.2.9.1.2' => '/dev/rmt/2cbn'
#      'enterprises.42.2.28.21.1.2.2.9.1.3' => 'AB8731'
#      'enterprises.42.2.28.21.1.2.2.9.1.4' => 'HP'
#      'enterprises.42.2.28.21.1.2.2.9.1.5' => 'Ultrium 1-SCSI'
#      'enterprises.42.2.28.21.1.2.2.9.1.6' => 'E1AV'
#      'enterprises.42.2.28.21.1.2.2.9.1.7' => 'The drive is having severe trouble reading.'
#      'system.sysUpTime.0' => 'Timeticks: (1) 0:00:00.01'
#  }
#
#############################################
#  AFTER CALLING last_snmp(,3):
#
#   '2.1.1' => 'TapeAlert'
#   '2.1.2' => 4
#   '2.1.3' => 'The tape drive is having problems reading data. No data has been.lost, but there has been a reduction in the performance of the.tape.'
#   '2.1.5' => 'ns-east-107'
#   '2.1.6' => 'Thu Nov 20 12:37:54 2003.'
#   '9.1.1' => 'SSC2 1'
#   '9.1.2' => '/dev/rmt/2cbn'
#   '9.1.3' => 'AB8731'
#   '9.1.4' => 'HP'
#   '9.1.5' => 'Ultrium 1-SCSI'
#   '9.1.6' => 'E1AV'
#   '9.1.7' => 'The drive is having severe trouble reading.'
#   'DATE' => '2003-11-22'
#   'DEVICE' => 'ns-east-107.East.Sun.COM'
#   'IP' => '10.8.11.107'
#   'TIME' => '10:32:03'
#   'snmpTrap.snmpTrapOID.0' => 'OID: enterprises.42.2.28.21.1.2.2.9.0.1'
#   'system.sysUpTime.0' => 'Timeticks: (1) 0:00:00.01'
#



1;
